#!/usr/bin/python
import threading, datetime, time, logging, os, os.path, minimalmodbus, requests

from apscheduler.scheduler import Scheduler
sched = Scheduler()
sched.start()

logging.basicConfig()

wn = minimalmodbus.Instrument('/dev/ttyUSB0', 1)
wn.serial.baudrate = 9600
wattson2 = minimalmodbus.Instrument("/dev/ttyUSB0", 2)
wattson2.serial.baudrate = 9600

def read_meter():
    #try:
        LEG1 = round(wn.read_float(1011), 3) # real pwr A
        LEG2 = round(wn.read_float(1013), 3) # real pwr B
        GENW = round(wn.read_float(1015), 3) # PV Watts
        CONS = round(wn.read_float(1009), 3)
        IMPA = round(wn.read_float(1107), 3)
        IMPB = round(wn.read_float(1109), 3)
        PVWH = round(wn.read_float(1111), 3)
        NETA = round(wn.read_float(1101), 3)
        NETB = round(wn.read_float(1103), 3)
        EXPA = round(wn.read_float(1117), 3)
        EXPB = round(wn.read_float(1119), 3)
        TECO = round(wn.read_float(1001), 1)
        L1PF = round(wn.read_float(1141), 4)
        L2PF = round(wn.read_float(1143), 4)
        PVPF = round(wn.read_float(1145), 4)
        VOLT = round(wn.read_float(1025), 3)
        PAMP = round(wn.read_float(1167), 2) # PV current
        DMND = round(wn.read_float(1169), 2)
        DMIN = round(wn.read_float(1171), 2)
        DMAX = round(wn.read_float(1173), 2)
        DMDA = round(wn.read_float(1177), 2)
        DMDB = round(wn.read_float(1179), 2)
        DMAP = round(wn.read_float(1175), 2)

        TOTL = LEG1 + LEG2
        IMPT = IMPA + IMPB                     # Import total kWh
        EXPT = EXPA + EXPB                     # Export total kWh
        NETT = IMPT - EXPT

        IMPW = 0
        if TOTL > 0:
            IMPW = TOTL

        DRCT = CONS - IMPW
        EXPW = GENW - DRCT

        meas = {
                'LEG1':LEG1,
                'LEG2':LEG2,
                'GENW':GENW,
                'PVWH':PVWH,
                'CONS':CONS,
                'TOTL':TOTL,
                'TECO':TECO,
                'IMPT':IMPT,
                'EXPT':EXPT,
                'NETT':NETT,
                'VOLT':VOLT,
                'L1PF':L1PF,
                'L2PF':L2PF,
                'PVPF':PVPF,
                'DRCT':DRCT,
                'IMPW':IMPW,
                'EXPW':EXPW,
                'DMND':DMND,
                'DMIN':DMIN,
                'DMAX':DMAX,
                'DMDA':DMDA,
                'DMDB':DMDB,
                'DMAP':DMAP,
                'PAMP':PAMP
               }

        if GENW < 50:
            meas['PAMP'] = 0

        tmpData = []
        for key, val in meas.items():
            tmpData.append('{} value={} {}'.format(key, val, int(time.time())))
            Data = '\n'.join(tmpData)

        #print Data
        requests.post('http://192.168.1.61:8086/write?db=energy&precision=s', data=Data)
        requests.post('http://140.174.6.17:8086/write?db=energy&precision=s', data=Data)
    #except:
        #pass

def read_water_htr():
    #try:
        WHWH = wattson2.read_float(768)        # west water htr kWh
        WHTR = wattson2.read_float(810) * 1000 #                W
        meas = {'WHTR':WHTR, 'WHWH':WHWH}

        tmpData = []
        for key, val in meas.items():
            tmpData.append('{} value={} {}'.format(key, val, int(time.time())))
            Data = '\n'.join(tmpData)

        requests.post('http://192.168.1.61:8086/write?db=energy&precision=s', data=Data)
        requests.post('http://140.174.6.17:8086/write?db=energy&precision=s', data=Data)
    #except:
        #pass


def send_to_pvo():
    CONS = round(wn.read_float(1009), 3)
    VOLT = round(wn.read_float(1019), 2)
    GENW = round(wn.read_float(1015), 0)
    if GENW < 50:
        GENW = 0

    DATE = time.strftime('%Y%m%d')
    TIME = time.strftime('%R')
    Headers = {'X-Pvoutput-Apikey':'aef5398a30db5747abf5148c64c00499fe4f9c74', 'X-Pvoutput-SystemId':'17152',}
    Data = {'d':DATE, 't':TIME, 'v2':GENW, 'v4':CONS, 'v6':VOLT}
    requests.post('http://pvoutput.org/service/r2/addstatus.jsp', headers=Headers, data=Data)
    
# write "end-of-day" values to a text file.
def wrt_daily_data():
    IMPA = round(wn.read_float(1107), 1)
    IMPB = round(wn.read_float(1109), 1)
    EXPA = round(wn.read_float(1117), 1)
    EXPB = round(wn.read_float(1119), 1)
    NETA = round(wn.read_float(1101), 2)             # net A kWh
    NETB = round(wn.read_float(1103), 2)             # net B kWh
    PVWH = round(wn.read_float(1105), 1)             # net C kWh (gend)
    CONS = round(wn.read_float(1009), 3)             # net A+B+C
    WHTR = round(wattson2.read_float(810), 2) * 1000 # west wtr htr W
    WHWH = round(wattson2.read_float(836), 3)        # water htr kWh
    IMPT = IMPA + IMPB                               # import total kWh
    EXPT = EXPA + EXPB                               # export total kWh
    NETT = IMPT - EXPT                               # net total kWh


    with open('/data/meter_data_daily.txt', 'a+') as daily_data:
	daily_data.write('{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n'.format(time.strftime('%Y%m%d %X'),
    IMPA,
    IMPB,
    IMPT,
    EXPA,
    EXPB,
    EXPT,
    NETA,
    NETB,
    PVWH,
    CONS,
    NETT,
    WHWH))
        
    # reset accumulated totals to zero
    wn.write_register(1619, 1, 0)

    with open('/tmp/pv.night', 'w') as pv_night:
        pv_night.write('0\n')

# wrap column titles in two lines of dashes to separate the monthly totals.
def wrt_monthly_separator():
    with open('/data/meter_data_daily.txt', 'a+') as daily_data:

        daily_data.write('{}\n'.format('-' * 119))
        daily_data.write('\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\t{}\n'.format
        ('',
         '',
         'IMPA',
         'IMPB',
         'IMPT',
         'EXPA',
         'EXPB',
         'EXPT',
         'NETA',
         'NETB',
         'PVWH',
         'CONS',
         'NETT',
         'WHKWH'))
        
        daily_data.write('{}\n'.format('-' * 119))

def main():
    sched.add_cron_job(read_meter, second='1,6,11,16,21,26,31,36,41,46,51,56')
    sched.add_cron_job(read_water_htr, second='3,8,13,18,23,28,33,38,43,48,53,58')
    sched.add_cron_job(send_to_pvo, minute='*/5', second='2')
    sched.add_cron_job(wrt_daily_data, hour='23', minute='59', second='52')
    sched.add_cron_job(wrt_monthly_separator, day='1', hour='0', minute='0', second='1')

    while True:
       time.sleep(1)

if __name__ == "__main__":
    main()
